/*
 * BQ25792.c
 *
 *  Created on: Jun 28, 2021
 *      Author: Daniel H.
 */

#include "bq25792.h"
#include "stdbool.h"
#include "i2c.h"

#define I2C_TIMEOUT_MS 100

BQ25792_Device BQ25792;

void bq25792_Init(I2C_HandleTypeDef* hi2c, struct BQ25792_Device** Device)
{
	*Device = &BQ25792;
	BQ25792.addr 	= ADDR_BQ25792;
	BQ25792.hi2c 	= hi2c;

	// Minimal settings to drive the charger

	while(HAL_I2C_IsDeviceReady(BQ25792.hi2c, BQ25792.addr, 5, I2C_TIMEOUT_MS) != HAL_OK)
	{
	  HAL_Delay(100);
	  HAL_GPIO_TogglePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin);
	}

	bq25792_set_charging(BQ25792_OFF);

	// Disable WatchDog
	uint16_t register_value = 0x88;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_CONTROL_1, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
	  Error_Handler();
	}

	/* Disable STAT Pin*/
	if(HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_CONTROL_4, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	register_value &= ~(1 << 4);
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_CONTROL_4, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Disable ILIM Pin*/
	register_value = 0x14;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_CONTROL_5, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Set Precharge current to 2A ILIM Pin*/
	register_value = 0xF2;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)PRECHARGE_CONTROL, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 2, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Set IINDPM current to 2A - 10mA steps */
	register_value = __REV16(330);
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)INPUT_CURRENT_LIMIT_MSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 2, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* VINDPM - 100mV steps*/
	register_value = 45;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)INPUT_VOLTAGE_LIMIT, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 2, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Set Charging Current 2A - 10mA steps */
	register_value = __REV16(50);
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGE_CURRENT_LIMIT_MSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 2, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Set Charging Voltage 16.8V - 10mV steps*/
	register_value = __REV16(1680);
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGE_VOLTAGE_LIMIT_MSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 2, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Enable ADC */
	register_value = 0xB0;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)ADC_CONTROL, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Set Term Current 80mA - 40mA steps (range: 40mA - 1000mA)*/
	register_value = 0x02;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)TERMINATION_CONTROL, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Mask INT*/
	register_value = 0xFF;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_MASK_0, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Mask INT*/
	register_value = 0x45;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_MASK_1, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Mask INT*/
	register_value = 0xFF;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_MASK_2, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Mask INT*/
	register_value = 0xFF;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_MASK_3, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Mask INT*/
	register_value = 0xFF;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)FAULT_MASK_0, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Mask INT*/
	register_value = 0xFF;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)FAULT_MASK_1, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	/* Ignore TS - Experimental State*/
	register_value = 0x55;
	if(HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)NTC_CONTROL_1, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&register_value, 1, I2C_TIMEOUT_MS) != HAL_OK)
	{
		Error_Handler();
	}

	bq25792_update_state(&BQ25792);
}


static int bq25792_get_vbat_adc()
{
	int ret;
	int adc_lsb = 0, adc_msb = 0;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)VBAT_ADC_MSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_msb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)VBAT_ADC_LSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_lsb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	 return adc_msb << 8 | adc_lsb;
 }

static int bq25792_get_vbus_adc()
{
	int ret;
	int adc_lsb = 0, adc_msb = 0;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)VBUS_ADC_MSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_msb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)VBUS_ADC_LSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_lsb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	return adc_msb << 8 | adc_lsb;
}

static int bq25792_get_ibus_adc()
{
	int ret;
	int adc_lsb = 0, adc_msb = 0;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)IBUS_ADC_MSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_msb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)IBUS_ADC_LSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_lsb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	return adc_msb << 8 | adc_lsb;
}

static int bq25792_get_ibat_adc()
{
	int ret;
	int adc_lsb = 0, adc_msb = 0;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)IBAT_ADC_MSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_msb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)IBAT_ADC_LSB, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&adc_lsb, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	return adc_msb << 8 | adc_lsb;
}

int bq25792_update_state()
{
	struct BQ25792_state new_state;
	int chrg_stat_0 = 0, chrg_stat_1 = 0, chrg_stat_2 = 0, chrg_stat_3 = 0, chrg_stat_4 = 0;
	int chrg_ctrl_0 = 0, fault_0 = 0, fault_1 = 0;
	int charger_flag_1 = 0;
	int ret = 0;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_STATUS_0, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&chrg_stat_0, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_STATUS_1, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&chrg_stat_1, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_STATUS_2, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&chrg_stat_2, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_CONTROL_0, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&chrg_ctrl_0, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_STATUS_4, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&chrg_stat_4, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)FAULT_FLAG_0, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&fault_0, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)FAULT_FLAG_1, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&fault_1, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_STATUS_3, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&chrg_stat_3, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_FLAG_1, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&charger_flag_1, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	new_state.vbus_status = chrg_stat_0 & BQ25792_VBUS_PRESENT;
	new_state.online = chrg_stat_0 & BQ25792_PG_STAT;

	if (chrg_ctrl_0 & BQ25792_CHRG_EN)
		new_state.chrg_status = chrg_stat_1 & BQ25792_CHG_STAT_MSK;
	else
		new_state.chrg_status = BQ25792_NOT_CHRGING;

	new_state.chrg_type 		= chrg_stat_1 & BQ25792_VBUS_STAT_MSK;
	new_state.vbat_present		= chrg_stat_2 & BQ25792_VBAT_PRESENT;
	new_state.temp 				= chrg_stat_4 & BQ25792_TEMP_MASK;
	new_state.fault_0 			= fault_0;
	new_state.fault_1 			= fault_1;
	new_state.charger_flag_1 	= charger_flag_1;
	new_state.vbat_adc 			= bq25792_get_vbat_adc();
	new_state.vbus_adc 			= bq25792_get_vbus_adc();
	new_state.ibat_adc 			= bq25792_get_ibat_adc();
	new_state.ibus_adc 			= bq25792_get_ibus_adc();

	ret = 	BQ25792.state.chrg_status 		!= new_state.chrg_status 	||
			BQ25792.state.online 			!= new_state.online         ||
			BQ25792.state.temp 				!= new_state.temp   		||
			BQ25792.state.fault_0 			!= new_state.fault_0 		||
			BQ25792.state.fault_1 			!= new_state.fault_1 		||
			BQ25792.state.charger_flag_1 	!= new_state.charger_flag_1 ||
			BQ25792.state.chrg_type 		!= new_state.chrg_type 		||
			BQ25792.state.vbat_adc 			!= new_state.vbat_adc 		||
			BQ25792.state.vbus_adc 			!= new_state.vbus_adc 		||
			BQ25792.state.ibat_adc 			!= new_state.ibat_adc;

	if(ret)
	  BQ25792.state = new_state;

	return ret;
}

int bq25792_set_charging(uint8_t OnOff)
{
	int ret;
	int prev_value = 0;

	ret = HAL_I2C_Mem_Read(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_CONTROL_0, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&prev_value, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	if(OnOff)
		prev_value |= BQ25792_EN_CHARGE;
	else
		prev_value &= ~BQ25792_EN_CHARGE;

	ret = HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)CHARGER_CONTROL_0, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&prev_value, 1, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
		return ret;

	return 0;
}

int bq25792_set_vindpm(uint16_t value)
{
	int ret;

	ret = HAL_I2C_Mem_Write(BQ25792.hi2c, BQ25792.addr, (uint16_t)INPUT_VOLTAGE_LIMIT, I2C_MEMADD_SIZE_8BIT, (uint8_t*)&value, 2, I2C_TIMEOUT_MS);
	if(ret != HAL_OK)
	{
		return ret;
	}

	return 0;
}
